package model;

import java.awt.Component;
import java.util.EventObject;

import javax.swing.DefaultCellEditor;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.event.CellEditorListener;
import javax.swing.table.TableCellEditor;

import util.ComboString;

/**
 * @author Nobuo Tamemasa
 * @author Mickal Martin Nevot
 * 
 * @version 1.0.0 11/09/1998
 * @version 1.1.0 28/03/2011
 * @version 1.2.0
 */
public class MultiEditor implements TableCellEditor {
	private final static int COMBO = 0;

	private final static int BOOLEAN = 1;

	private final static int STRING = 2;

	private final static int NUM_EDITOR = 3;

	private DefaultCellEditor[] cellEditors;
	private JComboBox comboBox;
	private int flg;

	public MultiEditor() {
		this.cellEditors = new DefaultCellEditor[NUM_EDITOR];
		//----------------------------------------------------
		this.comboBox = new JComboBox();
		this.comboBox.addItem("true");
		this.comboBox.addItem("false");
		this.cellEditors[COMBO] = new DefaultCellEditor(this.comboBox);
		//----------------------------------------------------
		JCheckBox checkBox = new JCheckBox();
		checkBox.setHorizontalAlignment(JLabel.CENTER);
		this.cellEditors[BOOLEAN] = new DefaultCellEditor(checkBox);
		//----------------------------------------------------
		JTextField textField = new JTextField();
		this.cellEditors[STRING] = new DefaultCellEditor(textField);
		//----------------------------------------------------
		this.flg = COMBO;
	}

	public Component getTableCellEditorComponent(JTable table, Object value,
			boolean isSelected, int row, int column) {
		System.err.println("getTableCellEditorComponent called:");
		System.err.println("   isSelected: " + isSelected);
		System.err.println("   row: " + row);
		System.err.println("   col: " + column);
		if (value instanceof ComboString) { // ComboString
			System.err.println("   ComboString");
			this.flg = COMBO;
			String str = (value == null) ? "" : value.toString();
			return this.cellEditors[COMBO].getTableCellEditorComponent(table, str,
					isSelected, row, column);
		} else if (value instanceof Boolean) { // Boolean
			System.err.println("   Boolean");
			this.flg = BOOLEAN;
			return this.cellEditors[BOOLEAN].getTableCellEditorComponent(table,
					value, isSelected, row, column);
		} else if (value instanceof String) { // String
			System.err.println("   String");
			this.flg = STRING;
			return this.cellEditors[STRING].getTableCellEditorComponent(table,
					value, isSelected, row, column);
		} else {
			System.err.println("   Trouble!");
		}
		return null;
	}

	public Object getCellEditorValue() {
		System.err.println("getCellEditorValue called");
		System.err.println("   this.flg = " + this.flg);
		switch (this.flg) {
		case COMBO:
			String str = (String) this.comboBox.getSelectedItem();
			return new ComboString(str);
		case BOOLEAN:
		case STRING:
			return this.cellEditors[this.flg].getCellEditorValue();
		default:
			return null;
		}
	}

	public Component getComponent() {
		return this.cellEditors[this.flg].getComponent();
	}

	public boolean stopCellEditing() {
		return this.cellEditors[this.flg].stopCellEditing();
	}

	public void cancelCellEditing() {
		this.cellEditors[this.flg].cancelCellEditing();
	}

	public boolean isCellEditable(EventObject anEvent) {
		return this.cellEditors[this.flg].isCellEditable(anEvent);
	}

	public boolean shouldSelectCell(EventObject anEvent) {
		return this.cellEditors[this.flg].shouldSelectCell(anEvent);
	}

	public void addCellEditorListener(CellEditorListener l) {
		this.cellEditors[this.flg].addCellEditorListener(l);
	}

	public void removeCellEditorListener(CellEditorListener l) {
		this.cellEditors[this.flg].removeCellEditorListener(l);
	}

	public void setClickCountToStart(int n) {
		this.cellEditors[this.flg].setClickCountToStart(n);
	}

	public int getClickCountToStart() {
		return this.cellEditors[this.flg].getClickCountToStart();
	}
}